@@ -5,7 +5,7 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 0.14
+;; Version: 1.0
 ;; Package-Requires: ((emacs "27.1") (compat "29.1.4.2"))
 ;; Homepage: https://github.com/minad/osm
 ;; Keywords: network, multimedia, hypermedia, mouse
@@ -69,6 +69,12 @@ A comma-separated specifies descending order of preference.  See also
 `url-mime-language-string'."
   :type 'string)
 
+(defcustom osm-search-server
+  "https://nominatim.openstreetmap.org"
+  "Server used to search for location names.
+The server must offer the nominatim.org API."
+  :type 'string)
+
 (defcustom osm-server-defaults
   '(:min-zoom 2
     :max-zoom 19
@@ -872,7 +878,7 @@ Should be at least 7 days according to the server usage policies."
         (osm--put-pin pins 'osm-poi (cadr pt) (cddr pt) (car pt))))
     pins))
 
-;; TODO The Bresenham algorithm used here to add the line segments to the tiles
+;; TODO: The Bresenham algorithm used here to add the line segments to the tiles
 ;; has the issue that lines which go along a tile border may be drawn only
 ;; partially. We can fix this by starting Bresenham at (x0±line width, y0±line
 ;; width).
@@ -966,9 +972,9 @@ xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>
 <image xlink:href='"
                           (if (eval-when-compile (> emacs-major-version 27))
                               (file-name-nondirectory file)
-                            ;; NOTE: On Emacs 27, :base-uri and embedding by
-                            ;; file path is not supported. Use the less
-                            ;; efficient base64 encoding.
+                            ;; On Emacs 27, :base-uri and embedding by file path
+                            ;; is not supported. Use the less efficient base64
+                            ;; encoding.
                             (svg--image-data
                              file
                              (if (member (file-name-extension file) '("jpg" "jpeg"))
@@ -1430,20 +1436,21 @@ When called interactively, call the function `osm-home'."
             (alist-get
              'display_name
              (osm--fetch-json
-              (format "https://nominatim.openstreetmap.org/reverse?format=json&accept-language=%s&zoom=%s&lat=%s&lon=%s"
-                      osm-search-language (min 18 (max 3 osm--zoom)) lat lon)))))))
+              (format "%s/reverse?format=json&accept-language=%s&zoom=%s&lat=%s&lon=%s"
+                      osm-search-server osm-search-language
+                      (min 18 (max 3 osm--zoom)) lat lon)))))))
 
 (defun osm--fetch-json (url)
   "Get json from URL."
   (osm--check-libraries)
-  (json-parse-string
-   (let ((default-process-coding-system '(utf-8-unix . utf-8-unix)))
-     (shell-command-to-string
-      (concat
-       "curl " osm-curl-options " "
-       (shell-quote-argument url))))
-   :array-type 'list
-   :object-type 'alist))
+  (with-temp-buffer
+    (let* ((default-process-coding-system '(utf-8-unix . utf-8-unix))
+           (status (apply #'call-process "curl" nil (current-buffer) nil
+                          `(,@(split-string-and-unquote osm-curl-options) ,url))))
+      (unless (eq status 0)
+        (error "Fetching %s exited with status %s" url status)))
+    (goto-char (point-min))
+    (json-parse-buffer :array-type 'list :object-type 'alist)))
 
 (defun osm--search (needle)
   "Globally search for NEEDLE and return the list of results."
@@ -1458,13 +1465,15 @@ When called interactively, call the function `osm-home'."
          ,@(mapcar #'string-to-number (alist-get 'boundingbox x)))))
    (or
     (osm--fetch-json
-     (format "https://nominatim.openstreetmap.org/search?format=json&accept-language=%s&q=%s"
-             osm-search-language (url-encode-url needle))))))
+     (format "%s/search?format=json&accept-language=%s&q=%s"
+             osm-search-server osm-search-language
+             (url-encode-url needle))))))
 
 ;;;###autoload
 (defun osm-search (needle &optional lucky)
-  "Globally search for NEEDLE and display the map.
-If the prefix argument LUCKY is non-nil take the first result and jump there."
+  "Globally search for NEEDLE on `osm-search-server' and display the map.
+If the prefix argument LUCKY is non-nil take the first result and jump there.
+See `osm-search-server' and `osm-search-language' for customization."
   (interactive
    (list
     (minibuffer-with-setup-hook
@@ -1481,7 +1490,7 @@ If the prefix argument LUCKY is non-nil take the first result and jump there."
                        (osm--sorted-table osm--search-history)
                        nil nil nil 'osm--search-history))
       current-prefix-arg))
-  ;; TODO add search bounded to current viewbox, bounded=1, viewbox=x1,y1,x2,y2
+  ;; TODO: Add search bounded to current viewbox, bounded=1, viewbox=x1,y1,x2,y2
   (let* ((results (or (osm--search needle) (error "No results for `%s'" needle)))
          (selected
           (or
@@ -1492,8 +1501,10 @@ If the prefix argument LUCKY is non-nil take the first result and jump there."
                   (when (and (eq completing-read-function #'completing-read-default)
                              (not (bound-and-true-p vertico-mode))
                              (not (bound-and-true-p icomplete-mode)))
-                    ;; Show matches immediately for default completion.
-                    (minibuffer-completion-help)))
+                    (let ((message-log-max nil)
+                          (inhibit-message t))
+                      ;; Show matches immediately for default completion.
+                      (minibuffer-completion-help))))
               (completing-read
                (format "Matches for '%s': " needle)
                (osm--sorted-table results)
@@ -1640,27 +1651,39 @@ If prefix ARG is given, store url as Elisp expression."
     (message "Saved in the kill ring: %s" url)))
 
 (cl-defun osm-add-server (server
-                          &rest props
+                          &rest properties
                           &key name description group url max-connections
                           max-zoom min-zoom download-batch subdomains copyright)
-  "Add SERVER with properties to `osm-server-list'.
+  "Add SERVER with PROPERTIES to `osm-server-list'.
 The properties are checked as keyword arguments.  See
 `osm-server-list' for documentation of the keywords."
   (declare (indent 1))
   (ignore name description group url max-connections max-zoom
           min-zoom download-batch subdomains copyright)
   (dolist (sym '(:name :description :group :url))
-    (unless (stringp (plist-get props sym))
+    (unless (stringp (plist-get properties sym))
       (error "Server property %s is required" sym)))
   (unless (and server (symbolp server))
-    (error "Server id must be a symbol."))
-  (setf (alist-get server osm-server-list) props)
+    (error "Server id must be a symbol"))
+  (setf (alist-get server osm-server-list) properties)
   nil)
 
 ;;;###autoload
 (when (>= emacs-major-version 28)
   (add-to-list 'browse-url-default-handlers '("\\`geo:" . osm)))
 
+;;;###autoload
+(eval-after-load 'ol
+  (lambda ()
+    (declare-function org-link-set-parameters "ol")
+    (declare-function osm--org-link-props "ext:osm")
+    (org-link-set-parameters
+     "geo"
+     :follow (lambda (link _) (osm (concat "geo:" link)))
+     :store (lambda ()
+              (when (derived-mode-p 'osm-mode)
+                (apply 'org-link-store-props (osm--org-link-props)))))))
+
 (dolist (sym (list #'osm-center #'osm-up #'osm-down #'osm-left #'osm-right
                    #'osm-up-up #'osm-down-down #'osm-left-left #'osm-right-right
                    #'osm-zoom-out #'osm-zoom-in #'osm-bookmark-set #'osm-gpx-hide
